package com.map;

public class NetworkLearning {
	
	// neural network to train
	private Network	network;
	
	// array of distances between neurons
	private double[] distance;
	
	// learning rate
	private double	learningRate = 0.1;
	// learning radius
	private double	learningRadius = 0.5;
	
	// squared learning radius multiplied by 2 (precalculated value to speed up computations)
	private double	squaredRadius2 = 2 * 7 * 7;
	
	// Learning rate
	public double getLearningRate(){
		return learningRate;
	}
	public void setLearningRate(double value){	
		learningRate = Math.max( 0.0, Math.min( 1.0, value ));		
	}
	
	// Learning radius
	public double getLearningRadius(){
		return learningRadius;
	}
	public void setLearningRadius(double value){
		learningRadius = Math.max( 0, Math.min( 1.0, value ) );
		squaredRadius2 = 2 * learningRadius * learningRadius;		
	}
	
	// Initializes a new instance
	public NetworkLearning( Network network )
	{
		this.network = network;
		
		// precalculate distances array
		int		neurons = network.layers.neuronsCount;
		double	deltaAlpha = Math.PI * 2.0 / neurons;
		double	alpha = deltaAlpha;
		distance = new double[neurons];
		distance[0] = 0.0;
		
		// calculate all distance values
		for ( int i = 1; i < neurons; i++ )
		{
			double dx = 0.5 * Math.cos( alpha ) - 0.5;
			double dy = 0.5 * Math.sin( alpha );

			distance[i] = dx * dx + dy * dy;

			alpha += deltaAlpha;
		}
	}
	
	// Runs learning iteration
	public double Run( double[] input )
	{
		// compute the network
		double error = 0.0;
		network.Compute( input );
		int winner = network.GetWinner( );
		
		// get layer of the network
		Layer layer = network.layers;
		
		// walk through all neurons of the layer
		for ( int j = 0, m = layer.getNeuronsCount(); j < m; j++ )
		{
			Neuron neuron = layer.neurons[j];
			
			// update factor
			double factor = Math.exp( - distance[Math.abs( j - winner )] / squaredRadius2 );
			
			// update weight of the neuron
			for ( int i = 0, n = neuron.getInputsCount(); i < n; i++ )
			{
				// calculate the error
				double e = ( input[i] - neuron.weights[i] ) * factor;				
				error += Math.abs( e );
				
				// update weight				
				neuron.weights[i] += e * learningRate;
			}
		}
		return error;
	}
	
	// Runs learning epoch
	public double RunEpoch( double[][] input )
	{
		double error = 0.0;
		
		// walk through all training samples
        for (int i = 1; i < input.length; i++)			
		{
            error += Run(input[i]);
		}
        
        // return summary error
		return error;
	}
}
